/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "udm_config.h"

#include <stdio.h>
#include <string.h>
#include "udm_uniconv.h"
#include "udm_unidata.h"
#include "udm_sgml.h"

/*
  Special characters:
  0x22 0010.0010 "
  0x26 0010.0110 &
  0x3C 0011.1100 <
  0x3E 0011.1110 >
       001?.???0
  
  0xE1 1110.0001 Mask
  0x20 0010.0000 Result
  
*/

#define IS_SPECIAL(x) ((x) == '"' || (x) == '&' || (x) == '<' || (x) == '>')

#define IS_SPECIAL_QUICK(x) ((((x) & 0xE1) == 0x20) && IS_SPECIAL(x))


static int
udm_mb_wc_ascii(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t *wc,
                const unsigned char *str,
                const unsigned char *end,
                int flags)
{
  if (*str == '&' && (flags & UDM_RECODE_HTML_SPECIAL))
    return UdmSGMLScan(wc, str, end);
  if (*str > 0x7F)
    return UDM_CHARSET_ILSEQ;
  *wc= *str;
  return 1;
}


static int
udm_wc_mb_ascii(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t wc,
                unsigned char *s, unsigned char *e,
                int flags)
{
  if (wc > 0x7F)
    return UDM_CHARSET_ILUNI;

  *s= wc;

   if ((flags & UDM_RECODE_HTML_SPECIAL) && IS_SPECIAL_QUICK(*s))
     return UDM_CHARSET_ILUNI;
  return 1;
}


static int
udm_mb_wc_latin1(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t *wc,
                 const unsigned char *str, const unsigned char *end,
                 int flags)
{
  if (*str == '&' && (flags & UDM_RECODE_HTML_SPECIAL))
    return UdmSGMLScan(wc, str, end);
  *wc= *str;
  return 1;
}


static int
udm_wc_mb_latin1(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t wc,
                 unsigned char *s, unsigned char *e,
                 int flags)
{
  if (wc > 0xFF)
    return UDM_CHARSET_ILUNI;

  *s= wc;

   if ((flags & UDM_RECODE_HTML_SPECIAL) && IS_SPECIAL_QUICK(*s))
     return UDM_CHARSET_ILUNI;
  return 1;
}


static int
udm_mb_wc_8bit(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t *wc,
               const unsigned char *str, const unsigned char *end,
               int flags)
{
  if (*str == '&' && (flags & UDM_RECODE_HTML_SPECIAL))
    return UdmSGMLScan(wc, str, end);
  *wc= cs->tab_to_uni[*str];
  return (!wc[0] && str[0]) ? UDM_CHARSET_ILSEQ : 1;
}


static int
udm_wc_mb_8bit(udm_mbstate_t *mbstate, UDM_CHARSET *cs, udm_wc_t wc,
               unsigned char *s, unsigned char *e,
               int flags)
{
  UDM_UNI_IDX *idx;
     
  for(idx=cs->tab_from_uni; idx->tab ; idx++)
  {
    if(idx->from <= wc && idx->to >= wc)
    {
      s[0]=idx->tab[wc - idx->from];
      if ((flags & UDM_RECODE_HTML_SPECIAL) && IS_SPECIAL_QUICK(*s))
        return UDM_CHARSET_ILUNI;
      return (!s[0] && wc) ? UDM_CHARSET_ILUNI : 1;
    }
  }
  return UDM_CHARSET_ILUNI;
}


static inline int
scan_one_char(UDM_UNIDATA *unidata, UDM_CHARSET *cs,
              const char *str, const char *strend, int *ctype, int flags)
{
  int wclen;
  if (*str == '&' && (flags & UDM_RECODE_HTML_IN))
  {
    udm_wc_t wc;
    wclen= UdmSGMLScan(&wc, (const unsigned char*) str, (const unsigned char*) strend);
    *ctype= UdmUniCType(unidata, wc);
  }
  else
  {
    wclen= 1;
    *ctype= cs->ctype[(unsigned char) *str];
  }
  if (*ctype == UDM_UNI_DIGIT || *ctype == UDM_UNI_CJK)
    *ctype= UDM_UNI_LETTER;
  return wclen;
}


static const char *
UdmStrGetSepToken8bit(UDM_UNIDATA *unidata, UDM_CHARSET *cs,
                      const char *str, const char *strend,
                      const char **last, int *ctype0, int flags)
{
  const char *beg;
  int ctype;

  if(str == NULL && (str= *last) == NULL)
    return NULL;

  if ((beg= str) >= strend)
    return NULL;

  str+= scan_one_char(unidata, cs, str, strend, ctype0, flags);
  for ( ; str < strend; )
  {
    int wclen= scan_one_char(unidata, cs, str, strend, &ctype, flags);
    if (*ctype0 != ctype)
      break;
    str+= wclen;
  }

  *last= str;
  return beg;
}


#if 0
/* This does not unescape HTML entities */
static void
UdmStrToLowerASCII(UDM_UNIDATA *uni_plane,
                   UDM_CHARSET *cs,
                   char *src, size_t srclen)
{
  const char *end= src + srclen;
  for ( ; src < end ; src++)
  {
    if (*src >= 'A' && *src <= 'Z')
      *src= *src - 'A' + 'a';
  }
}
#endif


UDM_CHARSET_HANDLER udm_charset_handler_8bit=
{
  udm_mb_wc_8bit,
  udm_wc_mb_8bit,
  UdmStrToLower8bit,
  UdmStrGetSepToken8bit,
  UdmGetWordMB,
  UdmCRC32LCase8bit,
  UdmWellFormedLengthGeneric
};


UDM_CHARSET_HANDLER udm_charset_handler_usascii=
{
  udm_mb_wc_ascii,
  udm_wc_mb_ascii,
  UdmStrToLower8bit,
  UdmStrGetSepToken8bit,
  UdmGetWordMB,
  UdmCRC32LCase8bit,
  UdmWellFormedLengthGeneric
};


UDM_CHARSET_HANDLER udm_charset_handler_latin1=
{
  udm_mb_wc_latin1,
  udm_wc_mb_latin1,
  UdmStrToLower8bit,
  UdmStrGetSepToken8bit,
  UdmGetWordMB,
  UdmCRC32LCase8bit,
  UdmWellFormedLengthGeneric
};
