/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "udm_config.h"
#include <string.h>

#include "udm_common.h"
#include "udm_uniconv.h"
#include "uconv-8bit.h"
#include "udm_utils.h"


UDM_CHARSET udm_charset_usascii=
{
  UDM_CHARSET_US_ASCII,
  &udm_charset_handler_usascii,
  "us-ascii", "ascii", NULL,
  UDM_CHARSET_WESTERN,
  ctype_us_ascii,
  tab_us_ascii_uni,
  idx_uni_us_ascii
};


UDM_CHARSET udm_charset_latin1=
{
  UDM_CHARSET_8859_1,
  &udm_charset_handler_latin1,
  "iso-8859-1", "latin1", "LATIN1",
  UDM_CHARSET_WESTERN,
  ctype_iso_8859_1,
  tab_8859_1_uni,
  idx_uni_8859_1
};


static UDM_CHARSET udm_charset_iso8859_10=
{
  UDM_CHARSET_8859_10,
  &udm_charset_handler_8bit,
  "iso-8859-10", NULL, "LATIN6",
  UDM_CHARSET_NORDIC,
  ctype_iso_8859_10,
  tab_8859_10_uni,
  idx_uni_8859_10
};


static UDM_CHARSET udm_charset_iso8859_11=
{
  UDM_CHARSET_8859_11,
  &udm_charset_handler_8bit,
  "iso-8859-11", NULL, NULL,
  UDM_CHARSET_THAI,
  ctype_iso_8859_11,
  tab_8859_11_uni,
  idx_uni_8859_11
};


static UDM_CHARSET udm_charset_iso8859_13=
{
  UDM_CHARSET_8859_13,
  &udm_charset_handler_8bit,
  "iso-8859-13", "latin7", "LATIN7",
  UDM_CHARSET_BALTIC,
  ctype_iso_8859_13,
  tab_8859_13_uni,
  idx_uni_8859_13
};


static UDM_CHARSET udm_charset_iso8859_14=
{
  UDM_CHARSET_8859_14,
  &udm_charset_handler_8bit,
  "iso-8859-14", NULL, "LATIN8",
  UDM_CHARSET_CELTIC,
  ctype_iso_8859_14,
  tab_8859_14_uni,
  idx_uni_8859_14
};


static UDM_CHARSET udm_charset_iso8859_15=
{
  UDM_CHARSET_8859_15,
  &udm_charset_handler_8bit,
  "iso-8859-15", NULL, "LATIN9",
  UDM_CHARSET_WESTERN,
  ctype_iso_8859_15,
  tab_8859_15_uni,
  idx_uni_8859_15
};


static UDM_CHARSET udm_charset_iso8859_16=
{
  UDM_CHARSET_8859_16,
  &udm_charset_handler_8bit,
  "iso-8859-16", NULL, "LATIN10",
  UDM_CHARSET_CENTRAL,
  ctype_iso_8859_16,
  tab_8859_16_uni,
  idx_uni_8859_16
};


static UDM_CHARSET udm_charset_iso8859_2=
{
  UDM_CHARSET_8859_2,
  &udm_charset_handler_8bit,
  "iso-8859-2", "latin2", "LATIN2",
  UDM_CHARSET_CENTRAL,
  ctype_iso_8859_2,
  tab_8859_2_uni,
  idx_uni_8859_2
};


static UDM_CHARSET udm_charset_iso8859_3=
{
  UDM_CHARSET_8859_3,
  &udm_charset_handler_8bit,
  "iso-8859-3", NULL, "latin3",
  UDM_CHARSET_SOUTHERN,
  ctype_iso_8859_3,
  tab_8859_3_uni,
  idx_uni_8859_3
};


static UDM_CHARSET udm_charset_iso8859_4=
{
  UDM_CHARSET_8859_4,
  &udm_charset_handler_8bit,
  "iso-8859-4", NULL, "latin4",
  UDM_CHARSET_BALTIC,
  ctype_iso_8859_4,
  tab_8859_4_uni,
  idx_uni_8859_4
};


static UDM_CHARSET udm_charset_iso8859_5=
{
  UDM_CHARSET_8859_5,
  &udm_charset_handler_8bit,
  "iso-8859-5", NULL, "ISO_8858_5",
  UDM_CHARSET_CYRILLIC,
  ctype_iso_8859_5,
  tab_8859_5_uni,
  idx_uni_8859_5
};


static UDM_CHARSET udm_charset_iso8859_6=
{
  UDM_CHARSET_8859_6,
  &udm_charset_handler_8bit,
  "iso-8859-6", NULL, "ISO_8859_6",
  UDM_CHARSET_ARABIC,
  ctype_iso_8859_6,
  tab_8859_6_uni,
  idx_uni_8859_6
};


static UDM_CHARSET udm_charset_iso8859_7=
{
  UDM_CHARSET_8859_7,
  &udm_charset_handler_8bit,
  "iso-8859-7", "greek", "ISO_8859_7",
  UDM_CHARSET_GREEK,
  ctype_iso_8859_7,
  tab_8859_7_uni,
  idx_uni_8859_7
};


static UDM_CHARSET udm_charset_iso8859_8=
{
  UDM_CHARSET_8859_8,
  &udm_charset_handler_8bit,
  "iso-8859-8", "hebrew", "ISO_8859_8",
  UDM_CHARSET_HEBREW,
  ctype_iso_8859_8,
  tab_8859_8_uni,
  idx_uni_8859_8
};


static UDM_CHARSET udm_charset_iso8859_9=
{
  UDM_CHARSET_8859_9,
  &udm_charset_handler_8bit,
  "iso-8859-9", NULL, "LATIN5",
  UDM_CHARSET_TURKISH,
  ctype_iso_8859_9,
  tab_8859_9_uni,
  idx_uni_8859_9
};


static UDM_CHARSET udm_charset_armscii8=
{
  UDM_CHARSET_ARMSCII_8,
  &udm_charset_handler_8bit,
  "armscii-8", "armscii8", NULL,
  UDM_CHARSET_ARMENIAN,
  ctype_armscii_8,
  tab_armscii_8_uni,
  idx_uni_armscii_8
};


static UDM_CHARSET udm_charset_cp1250=
{
  UDM_CHARSET_CP1250,
  &udm_charset_handler_8bit,
  "windows-1250", "cp1250", "WIN1250",
  UDM_CHARSET_CENTRAL,
  ctype_windows_1250,
  tab_cp1250_uni,
  idx_uni_cp1250
};


static UDM_CHARSET udm_charset_cp1251=
{
  UDM_CHARSET_CP1251,
  &udm_charset_handler_8bit,
  "windows-1251", "cp1251", "WIN1251",
  UDM_CHARSET_CYRILLIC,
  ctype_windows_1251,
  tab_cp1251_uni,
  idx_uni_cp1251
};


static UDM_CHARSET udm_charset_cp1252=
{
  UDM_CHARSET_CP1252,
  &udm_charset_handler_8bit,
  "windows-1252", "cp1252", "WIN1252",
  UDM_CHARSET_WESTERN,
  ctype_windows_1252,
  tab_cp1252_uni,
  idx_uni_cp1252
};


static UDM_CHARSET udm_charset_cp1253=
{
  UDM_CHARSET_CP1253,
  &udm_charset_handler_8bit,
  "windows-1253", NULL, "WIN1253",
  UDM_CHARSET_GREEK,
  ctype_windows_1253,
  tab_cp1253_uni,
  idx_uni_cp1253
};


static UDM_CHARSET udm_charset_cp1254=
{
  UDM_CHARSET_CP1254,
  &udm_charset_handler_8bit,
  "windows-1254", NULL, "WIN1254",
  UDM_CHARSET_TURKISH,
  ctype_windows_1254,
  tab_cp1254_uni,
  idx_uni_cp1254
};


static UDM_CHARSET udm_charset_cp1255=
{
  UDM_CHARSET_CP1255,
  &udm_charset_handler_8bit,
  "windows-1255", NULL, "WIN1255",
  UDM_CHARSET_HEBREW,
  ctype_windows_1255,
  tab_cp1255_uni,
  idx_uni_cp1255
};


static UDM_CHARSET udm_charset_cp1256=
{
  UDM_CHARSET_CP1256,
  &udm_charset_handler_8bit,
  "windows-1256", "cp1256", "WIN1256",
  UDM_CHARSET_ARABIC,
  ctype_windows_1256,
  tab_cp1256_uni,
  idx_uni_cp1256
};


static UDM_CHARSET udm_charset_cp1257=
{
  UDM_CHARSET_CP1257,
  &udm_charset_handler_8bit,
  "windows-1257", "cp1257", "WIN1257",
  UDM_CHARSET_BALTIC,
  ctype_windows_1257,
  tab_cp1257_uni,
  idx_uni_cp1257
};


static UDM_CHARSET udm_charset_cp1258=
{
  UDM_CHARSET_CP1258,
  &udm_charset_handler_8bit,
  "windows-1258", NULL, "WIN1258",
  UDM_CHARSET_VIETNAMESE,
  ctype_windows_1258,
  tab_cp1258_uni,
  idx_uni_cp1258
};


static UDM_CHARSET udm_charset_cp437=
{
  UDM_CHARSET_CP437,
  &udm_charset_handler_8bit,
  "cp437", NULL, NULL,
  UDM_CHARSET_WESTERN,
  ctype_cp437,
  tab_cp437_uni,
  idx_uni_cp437
};


static UDM_CHARSET udm_charset_cp850=
{
  UDM_CHARSET_CP850,
  &udm_charset_handler_8bit,
  "cp850", "cp850", NULL,
  UDM_CHARSET_WESTERN,
  ctype_cp850,
  tab_cp850_uni,
  idx_uni_cp850
};


static UDM_CHARSET udm_charset_cp852=
{
  UDM_CHARSET_CP852,
  &udm_charset_handler_8bit,
  "cp852", "cp852", NULL,
  UDM_CHARSET_CENTRAL,
  ctype_cp852,
  tab_cp852_uni,
  idx_uni_cp852
};


static UDM_CHARSET udm_charset_cp855=
{
  UDM_CHARSET_CP855,
  &udm_charset_handler_8bit,
  "cp855", NULL, NULL,
  UDM_CHARSET_CYRILLIC,
  ctype_cp855,
  tab_cp855_uni,
  idx_uni_cp855
};


static UDM_CHARSET udm_charset_cp857=
{
  UDM_CHARSET_CP857,
  &udm_charset_handler_8bit,
  "cp857", NULL, NULL,
  UDM_CHARSET_TURKISH,
  ctype_cp857,
  tab_cp857_uni,
  idx_uni_cp857
};


static UDM_CHARSET udm_charset_cp860=
{
  UDM_CHARSET_CP860,
  &udm_charset_handler_8bit,
  "cp860", NULL, NULL,
  UDM_CHARSET_WESTERN,
  ctype_cp860,
  tab_cp860_uni,
  idx_uni_cp860
};


static UDM_CHARSET udm_charset_cp861=
{
  UDM_CHARSET_CP861,
  &udm_charset_handler_8bit,
  "cp861", NULL, NULL,
  UDM_CHARSET_ICELANDIC,
  ctype_cp861,
  tab_cp861_uni,
  idx_uni_cp861
};


static UDM_CHARSET udm_charset_cp862=
{
  UDM_CHARSET_CP862,
  &udm_charset_handler_8bit,
  "cp862", NULL, NULL,
  UDM_CHARSET_HEBREW,
  ctype_cp862,
  tab_cp862_uni,
  idx_uni_cp862
};


static UDM_CHARSET udm_charset_cp863=
{
  UDM_CHARSET_CP863,
  &udm_charset_handler_8bit,
  "cp863", NULL, NULL,
  UDM_CHARSET_WESTERN,
  ctype_cp863,
  tab_cp863_uni,
  idx_uni_cp863
};


static UDM_CHARSET udm_charset_cp864=
{
  UDM_CHARSET_CP864,
  &udm_charset_handler_8bit,
  "cp864", NULL, NULL,
  UDM_CHARSET_ARABIC,
  ctype_cp864,
  tab_cp864_uni,
  idx_uni_cp864
};


static UDM_CHARSET udm_charset_cp865=
{
  UDM_CHARSET_CP865,
  &udm_charset_handler_8bit,
  "cp865", NULL, NULL,
  UDM_CHARSET_NORDIC,
  ctype_cp865,
  tab_cp865_uni,
  idx_uni_cp865
};


static UDM_CHARSET udm_charset_cp866=
{
  UDM_CHARSET_CP866,
  &udm_charset_handler_8bit,
  "cp866", "cp866", "WIN866",
  UDM_CHARSET_CYRILLIC,
  ctype_cp866,
  tab_cp866_uni,
  idx_uni_cp866
};


static UDM_CHARSET udm_charset_cp869=
{
  UDM_CHARSET_CP869,
  &udm_charset_handler_8bit,
  "cp869", NULL, NULL,
  UDM_CHARSET_GREEK,
  ctype_cp869,
  tab_cp869_uni,
  idx_uni_cp869
};


static UDM_CHARSET udm_charset_cp874=
{
  UDM_CHARSET_CP874,
  &udm_charset_handler_8bit,
  "cp874", NULL, "WIN874",
  UDM_CHARSET_THAI,
  ctype_cp874,
  tab_cp874_uni,
  idx_uni_cp874
};


static UDM_CHARSET udm_charset_koi8r=
{
  UDM_CHARSET_KOI8_R,
  &udm_charset_handler_8bit,
  "koi8-r", "koi8r", "KOI8R",
  UDM_CHARSET_CYRILLIC,
  ctype_koi8_r,
  tab_koi8_r_uni,
  idx_uni_koi8_r
};


static UDM_CHARSET udm_charset_koi8u=
{
  UDM_CHARSET_KOI8_U,
  &udm_charset_handler_8bit,
  "koi8-u", "koi8u", "KOI8U",
  UDM_CHARSET_CYRILLIC,
  ctype_koi8_u,
  tab_koi8_u_uni,
  idx_uni_koi8_u
};


static UDM_CHARSET udm_charset_macarabic=
{
  UDM_CHARSET_MACARABIC,
  &udm_charset_handler_8bit,
  "MacArabic", NULL, NULL,
  UDM_CHARSET_ARABIC,
  ctype_MacArabic,
  tab_macarabic_uni,
  idx_uni_macarabic
};


static UDM_CHARSET udm_charset_macce=
{
  UDM_CHARSET_MACCE,
  &udm_charset_handler_8bit,
  "MacCE", "macce", NULL,
  UDM_CHARSET_CENTRAL,
  ctype_MacCE,
  tab_macce_uni,
  idx_uni_macce
};


static UDM_CHARSET udm_charset_maccroatian=
{
  UDM_CHARSET_MACCROATIAN,
  &udm_charset_handler_8bit,
  "MacCroatian", NULL, NULL,
  UDM_CHARSET_CENTRAL,
  ctype_MacCroatian,
  tab_maccroatian_uni,
  idx_uni_maccroatian
};


static UDM_CHARSET udm_charset_maccyrillic=
{
  UDM_CHARSET_MACCYRILLIC,
  &udm_charset_handler_8bit,
  "MacCyrillic", NULL, NULL,
  UDM_CHARSET_CYRILLIC,
  ctype_MacCyrillic,
  tab_maccyrillic_uni,
  idx_uni_maccyrillic
};


static UDM_CHARSET udm_charset_macgreek=
{
  UDM_CHARSET_MACGREEK,
  &udm_charset_handler_8bit,
  "MacGreek", NULL, NULL,
  UDM_CHARSET_GREEK,
  ctype_MacGreek,
  tab_macgreek_uni,
  idx_uni_macgreek
};


static UDM_CHARSET udm_charset_machebrew=
{
  UDM_CHARSET_MACHEBREW,
  &udm_charset_handler_8bit,
  "MacHebrew", NULL, NULL,
  UDM_CHARSET_HEBREW,
  ctype_MacHebrew,
  tab_machebrew_uni,
  idx_uni_machebrew
};


static UDM_CHARSET udm_charset_maciceland=
{
  UDM_CHARSET_MACICELAND,
  &udm_charset_handler_8bit,
  "MacIceland", NULL, NULL,
  UDM_CHARSET_ICELANDIC,
  ctype_MacIceland,
  tab_maciceland_uni,
  idx_uni_maciceland
};


static UDM_CHARSET udm_charset_macroman=
{
  UDM_CHARSET_MACROMAN,
  &udm_charset_handler_8bit,
  "MacRoman", "macroman", NULL,
  UDM_CHARSET_WESTERN,
  ctype_MacRoman,
  tab_macroman_uni,
  idx_uni_macroman
};


static UDM_CHARSET udm_charset_macromania=
{
  UDM_CHARSET_MACROMANIA,
  &udm_charset_handler_8bit,
  "MacRomania", NULL, NULL,
  UDM_CHARSET_CENTRAL,
  ctype_MacRomania,
  tab_macromania_uni,
  idx_uni_macromania
};


static UDM_CHARSET udm_charset_macthai=
{
  UDM_CHARSET_MACTHAI,
  &udm_charset_handler_8bit,
  "MacThai", NULL, NULL,
  UDM_CHARSET_THAI,
  ctype_MacThai,
  tab_macthai_uni,
  idx_uni_macthai
};


static UDM_CHARSET udm_charset_macturkish=
{
  UDM_CHARSET_MACTURKISH,
  &udm_charset_handler_8bit,
  "MacTurkish", NULL, NULL,
  UDM_CHARSET_TURKISH,
  ctype_MacTurkish,
  tab_macturkish_uni,
  idx_uni_macturkish
};


static UDM_CHARSET udm_charset_viscii=
{
  UDM_CHARSET_VISCII,
  &udm_charset_handler_8bit,
  "viscii", NULL, NULL,
  UDM_CHARSET_VIETNAMESE,
  ctype_viscii,
  tab_viscii_uni,
  idx_uni_viscii
};


static UDM_CHARSET udm_charset_geostd8=
{
  UDM_CHARSET_GEOSTD8,
  &udm_charset_handler_8bit,
  "geostd8", "geostd8", NULL,
  UDM_CHARSET_GEORGIAN,
  ctype_geostd8,
  tab_geostd8_uni,
  idx_uni_geostd8
};


static UDM_CHARSET *built_charsets2[]=
{
  &udm_charset_latin1,
  &udm_charset_iso8859_10,
  &udm_charset_iso8859_10,
  &udm_charset_iso8859_11,
  &udm_charset_iso8859_13,
  &udm_charset_iso8859_14,
  &udm_charset_iso8859_15,
  &udm_charset_iso8859_16,
  &udm_charset_iso8859_2,
  &udm_charset_iso8859_3,
  &udm_charset_iso8859_4,
  &udm_charset_iso8859_5,
  &udm_charset_iso8859_6,
  &udm_charset_iso8859_7,
  &udm_charset_iso8859_8,
  &udm_charset_iso8859_9,
  &udm_charset_armscii8,
  &udm_charset_cp1250,
  &udm_charset_cp1251,
  &udm_charset_cp1252,
  &udm_charset_cp1253,
  &udm_charset_cp1254,
  &udm_charset_cp1255,
  &udm_charset_cp1256,
  &udm_charset_cp1257,
  &udm_charset_cp1258,
  &udm_charset_cp437,
  &udm_charset_cp850,
  &udm_charset_cp852,
  &udm_charset_cp855,
  &udm_charset_cp857,
  &udm_charset_cp860,
  &udm_charset_cp861,
  &udm_charset_cp862,
  &udm_charset_cp863,
  &udm_charset_cp864,
  &udm_charset_cp865,
  &udm_charset_cp866,
  &udm_charset_cp869,
  &udm_charset_cp874,
  &udm_charset_koi8r,
  &udm_charset_koi8u,
  &udm_charset_macarabic,
  &udm_charset_macce,
  &udm_charset_maccroatian,
  &udm_charset_maccyrillic,
  &udm_charset_macgreek,
  &udm_charset_machebrew,
  &udm_charset_maciceland,
  &udm_charset_macroman,
  &udm_charset_macromania,
  &udm_charset_macthai,
  &udm_charset_macturkish,
  &udm_charset_usascii,
  &udm_charset_viscii,
  &udm_charset_geostd8,
  &udm_charset_utf8,
  &udm_charset_sys_int,
#ifdef HAVE_CHARSET_gb2312
  &udm_charset_gb2312,
#endif
#ifdef HAVE_CHARSET_big5
  &udm_charset_big5,
#endif
#ifdef HAVE_CHARSET_japanese
  &udm_charset_sjis,
#endif
#ifdef HAVE_CHARSET_euc_kr
  &udm_charset_euckr,
#endif
#ifdef HAVE_CHARSET_japanese
  &udm_charset_eucjp,
#endif
#ifdef HAVE_CHARSET_gbk
  &udm_charset_gbk,
#endif
#ifdef HAVE_CHARSET_gujarati
  &udm_charset_gujarati,
#endif
#ifdef HAVE_CHARSET_tscii
  &udm_charset_tscii,
#endif
#ifdef HAVE_CHARSET_japanese
  &udm_charset_iso2022jp,
#endif
  NULL
};



typedef struct
{
  const	char *name;
  UDM_CHARSET *cs;
} UDM_CHARSET_ALIAS;


/* Must be in alphabetic order */
static UDM_CHARSET_ALIAS alias[]=
{
  {"437",                  &udm_charset_cp437},
  {"850",                  &udm_charset_cp850},
  {"852",                  &udm_charset_cp852},
  {"855",                  &udm_charset_cp855},
  {"857",                  &udm_charset_cp857},
  {"860",                  &udm_charset_cp860},
  {"861",                  &udm_charset_cp861},
  {"862",                  &udm_charset_cp862},
  {"863",                  &udm_charset_cp863},
  {"864",                  &udm_charset_cp864},
  {"865",                  &udm_charset_cp865},
  {"866",                  &udm_charset_cp866},
  {"869",                  &udm_charset_cp869},
  {"874",                  &udm_charset_cp874},
  {"ansi_x3.4-1968",       &udm_charset_usascii},
  {"arabic",               &udm_charset_iso8859_6},
  {"armscii-8",            &udm_charset_armscii8},
  {"armscii8",             &udm_charset_armscii8},
  {"ascii",                &udm_charset_usascii},
  {"asmo-708",             &udm_charset_iso8859_6},
#ifdef HAVE_CHARSET_big5
  {"big-5",                &udm_charset_big5},
  {"big-five",             &udm_charset_big5},
  {"big5",                 &udm_charset_big5},
  {"bigfive",              &udm_charset_big5},
#endif
#ifdef HAVE_CHARSET_gb2312
  {"chinese",              &udm_charset_gb2312},
#endif
#ifdef HAVE_CHARSET_big5
  {"cn-big5",              &udm_charset_big5},
#endif
  {"cp-1250",              &udm_charset_cp1250},
  {"cp-1251",              &udm_charset_cp1251},
  {"cp-1252",              &udm_charset_cp1252},
  {"cp-1253",              &udm_charset_cp1253},
  {"cp-1254",              &udm_charset_cp1254},
  {"cp-1255",              &udm_charset_cp1255},
  {"cp-1256",              &udm_charset_cp1256},
  {"cp-1257",              &udm_charset_cp1257},
  {"cp-1258",              &udm_charset_cp1258},
  {"cp1250",               &udm_charset_cp1250},
  {"cp1251",               &udm_charset_cp1251},
  {"cp1252",               &udm_charset_cp1252},
  {"cp1253",               &udm_charset_cp1253},
  {"cp1254",               &udm_charset_cp1254},
  {"cp1255",               &udm_charset_cp1255},
  {"cp1256",               &udm_charset_cp1256},
  {"cp1257",               &udm_charset_cp1257},
  {"cp1258",               &udm_charset_cp1258},
  {"cp367",                &udm_charset_usascii},
  {"cp437",                &udm_charset_cp437},
  {"cp819",                &udm_charset_latin1},
  {"cp850",                &udm_charset_cp850},
  {"cp852",                &udm_charset_cp852},
  {"cp855",                &udm_charset_cp855},
  {"cp857",                &udm_charset_cp857},
  {"cp860",                &udm_charset_cp860},
  {"cp861",                &udm_charset_cp861},
  {"cp862",                &udm_charset_cp862},
  {"cp863",                &udm_charset_cp863},
  {"cp864",                &udm_charset_cp864},
  {"cp865",                &udm_charset_cp865},
  {"cp866",                &udm_charset_cp866},
  {"cp869",                &udm_charset_cp869},
  {"cp874",                &udm_charset_cp874},
#ifdef HAVE_CHARSET_gbk
  {"cp936",                &udm_charset_gbk},
#endif
#ifdef HAVE_CHARSET_big5
  {"cp950",                &udm_charset_big5},
#endif
  {"cs874",                &udm_charset_cp874},
  {"csascii",              &udm_charset_usascii},
#ifdef HAVE_CHARSET_big5
  {"csbig5",               &udm_charset_big5},
#endif
#ifdef HAVE_CHARSET_japanese
  {"cseucjp",              &udm_charset_eucjp},
#endif
#ifdef HAVE_CHARSET_euc_kr
  {"cseuckr",              &udm_charset_euckr},
#endif
#ifdef HAVE_CHARSET_gb2312
  {"csgb2312",             &udm_charset_gb2312},
#endif
  {"csibm866",             &udm_charset_cp866},
  {"csibm869",             &udm_charset_cp869},
#ifdef HAVE_CHARSET_gb2312
  {"csiso58gb231280",      &udm_charset_gb2312},
#endif
  {"csisolatin1",          &udm_charset_latin1},
  {"csisolatin2",          &udm_charset_iso8859_2},
  {"csisolatin3",          &udm_charset_iso8859_3},
  {"csisolatin4",          &udm_charset_iso8859_4},
  {"csisolatin5",          &udm_charset_iso8859_9},
  {"csisolatin6",          &udm_charset_iso8859_10},
  {"csisolatinarabic",     &udm_charset_iso8859_6},
  {"csisolatincyrillic",   &udm_charset_iso8859_5},
  {"csisolatingreek",      &udm_charset_iso8859_7},
  {"csisolatinhebrew",     &udm_charset_iso8859_8},
  {"cskoi8r",              &udm_charset_koi8r},
  {"csmacintosh",          &udm_charset_macroman},
  {"cspc850multilingual",  &udm_charset_cp850},
#ifdef HAVE_CHARSET_japanese
  {"csshiftjis",           &udm_charset_sjis},
#endif
  {"csviscii",             &udm_charset_viscii},
  {"cyrillic",             &udm_charset_iso8859_5},
  {"ecma-114",             &udm_charset_iso8859_6},
  {"ecma-118",             &udm_charset_iso8859_7},
  {"elot_928",             &udm_charset_iso8859_7},
#ifdef HAVE_CHARSET_japanese
  {"euc-jp",               &udm_charset_eucjp},
#endif
#ifdef HAVE_CHARSET_euc_kr
  {"euc-kr",               &udm_charset_euckr},
#endif
#ifdef HAVE_CHARSET_japanese
  {"eucjp",                &udm_charset_eucjp},
#endif
#ifdef HAVE_CHARSET_euc_kr
  {"euckr",                &udm_charset_euckr},
#endif
#ifdef HAVE_CHARSET_gb2312
  {"gb2312",               &udm_charset_gb2312},
  {"gb_2312-80",           &udm_charset_gb2312},
#endif
#ifdef HAVE_CHARSET_gbk
  {"gbk",                  &udm_charset_gbk},
#endif
  {"geo8-gov",             &udm_charset_geostd8},
  {"geostd8",              &udm_charset_geostd8},
  {"greek",                &udm_charset_iso8859_7},
  {"greek8",               &udm_charset_iso8859_7},
  {"hebrew",               &udm_charset_iso8859_8},
  {"ibm367",               &udm_charset_usascii},
  {"ibm437",               &udm_charset_cp437},
  {"ibm819",               &udm_charset_latin1},
  {"ibm850",               &udm_charset_cp850},
  {"ibm852",               &udm_charset_cp852},
  {"ibm855",               &udm_charset_cp855},
  {"ibm857",               &udm_charset_cp857},
  {"ibm860",               &udm_charset_cp860},
  {"ibm861",               &udm_charset_cp861},
  {"ibm862",               &udm_charset_cp862},
  {"ibm863",               &udm_charset_cp863},
  {"ibm864",               &udm_charset_cp864},
  {"ibm865",               &udm_charset_cp865},
  {"ibm866",               &udm_charset_cp866},
  {"ibm869",               &udm_charset_cp869},
  {"ibm874",               &udm_charset_cp874},
#ifdef HAVE_CHARSET_japanese
  {"iso-2022-jp",          &udm_charset_iso2022jp},
#endif
  {"iso-8859-1",           &udm_charset_latin1},
  {"iso-8859-10",          &udm_charset_iso8859_10},
  {"iso-8859-11",          &udm_charset_iso8859_11},
  {"iso-8859-13",          &udm_charset_iso8859_13},
  {"iso-8859-14",          &udm_charset_iso8859_14},
  {"iso-8859-15",          &udm_charset_iso8859_15},
  {"iso-8859-16",          &udm_charset_iso8859_16},
  {"iso-8859-2",           &udm_charset_iso8859_2},
  {"iso-8859-3",           &udm_charset_iso8859_3},
  {"iso-8859-4",           &udm_charset_iso8859_4},
  {"iso-8859-5",           &udm_charset_iso8859_5},
  {"iso-8859-6",           &udm_charset_iso8859_6},
  {"iso-8859-7",           &udm_charset_iso8859_7},
  {"iso-8859-8",           &udm_charset_iso8859_8},
  {"iso-8859-9",           &udm_charset_iso8859_9},
  {"iso-ir-100",           &udm_charset_latin1},
  {"iso-ir-101",           &udm_charset_iso8859_2},
  {"iso-ir-109",           &udm_charset_iso8859_3},
  {"iso-ir-110",           &udm_charset_iso8859_4},
  {"iso-ir-126",           &udm_charset_iso8859_7},
  {"iso-ir-127",           &udm_charset_iso8859_6},
  {"iso-ir-138",           &udm_charset_iso8859_8},
  {"iso-ir-144",           &udm_charset_iso8859_5},
  {"iso-ir-148",           &udm_charset_iso8859_9},
  {"iso-ir-157",           &udm_charset_iso8859_10},
  {"iso-ir-179",           &udm_charset_iso8859_13},
  {"iso-ir-199",           &udm_charset_iso8859_14},
  {"iso-ir-203",           &udm_charset_iso8859_15},
  {"iso-ir-226",           &udm_charset_iso8859_16},
#ifdef HAVE_CHARSET_gb2312
  {"iso-ir-58",            &udm_charset_gb2312},
#endif
  {"iso-ir-6",             &udm_charset_usascii},
  {"iso646-us",            &udm_charset_usascii},
  {"iso8859-1",            &udm_charset_latin1},
  {"iso8859-10",           &udm_charset_iso8859_10},
  {"iso8859-11",           &udm_charset_iso8859_11},
  {"iso8859-13",           &udm_charset_iso8859_13},
  {"iso8859-14",           &udm_charset_iso8859_14},
  {"iso8859-15",           &udm_charset_iso8859_15},
  {"iso8859-16",           &udm_charset_iso8859_16},
  {"iso8859-2",            &udm_charset_iso8859_2},
  {"iso8859-3",            &udm_charset_iso8859_3},
  {"iso8859-4",            &udm_charset_iso8859_4},
  {"iso8859-5",            &udm_charset_iso8859_5},
  {"iso8859-6",            &udm_charset_iso8859_6},
  {"iso8859-7",            &udm_charset_iso8859_7},
  {"iso8859-8",            &udm_charset_iso8859_8},
  {"iso8859-9",            &udm_charset_iso8859_9},
  {"iso_646.irv:1991",     &udm_charset_usascii},
  {"iso_8859-1",           &udm_charset_latin1},
  {"iso_8859-10",          &udm_charset_iso8859_10},
  {"iso_8859-10:1992",     &udm_charset_iso8859_10},
  {"iso_8859-11",          &udm_charset_iso8859_11},
  {"iso_8859-11:1992",     &udm_charset_iso8859_11},
  {"iso_8859-13",          &udm_charset_iso8859_13},
  {"iso_8859-14",          &udm_charset_iso8859_14},
  {"iso_8859-14:1998",     &udm_charset_iso8859_14},
  {"iso_8859-15",          &udm_charset_iso8859_15},
  {"iso_8859-15:1998",     &udm_charset_iso8859_15},
  {"iso_8859-16",          &udm_charset_iso8859_16},
  {"iso_8859-16:2000",     &udm_charset_iso8859_16},
  {"iso_8859-1:1987",      &udm_charset_latin1},
  {"iso_8859-2",           &udm_charset_iso8859_2},
  {"iso_8859-2:1987",      &udm_charset_iso8859_2},
  {"iso_8859-3",           &udm_charset_iso8859_3},
  {"iso_8859-3:1988",      &udm_charset_iso8859_3},
  {"iso_8859-4",           &udm_charset_iso8859_4},
  {"iso_8859-4:1988",      &udm_charset_iso8859_4},
  {"iso_8859-5",           &udm_charset_iso8859_5},
  {"iso_8859-5:1988",      &udm_charset_iso8859_5},
  {"iso_8859-6",           &udm_charset_iso8859_6},
  {"iso_8859-6:1987",      &udm_charset_iso8859_6},
  {"iso_8859-7",           &udm_charset_iso8859_7},
  {"iso_8859-7:1987",      &udm_charset_iso8859_7},
  {"iso_8859-8",           &udm_charset_iso8859_8},
  {"iso_8859-8:1988",      &udm_charset_iso8859_8},
  {"iso_8859-9",           &udm_charset_iso8859_9},
  {"iso_8859-9:1989",      &udm_charset_iso8859_9},
  {"koi8-r",               &udm_charset_koi8r},
  {"koi8-u",               &udm_charset_koi8u},
  {"koi8r",                &udm_charset_koi8r},
  {"koi8u",                &udm_charset_koi8u},
  {"l1",                   &udm_charset_latin1},
  {"l2",                   &udm_charset_iso8859_2},
  {"l3",                   &udm_charset_iso8859_3},
  {"l4",                   &udm_charset_iso8859_4},
  {"l5",                   &udm_charset_iso8859_9},
  {"l6",                   &udm_charset_iso8859_10},
  {"l7",                   &udm_charset_iso8859_13},
  {"l8",                   &udm_charset_iso8859_14},
  {"latin1",               &udm_charset_latin1},
  {"latin2",               &udm_charset_iso8859_2},
  {"latin3",               &udm_charset_iso8859_3},
  {"latin4",               &udm_charset_iso8859_4},
  {"latin5",               &udm_charset_iso8859_9},
  {"latin6",               &udm_charset_iso8859_10},
  {"latin7",               &udm_charset_iso8859_13},
  {"latin8",               &udm_charset_iso8859_14},
  {"mac",                  &udm_charset_macroman},
  {"macarabic",            &udm_charset_macarabic},
  {"macce",                &udm_charset_macce},
  {"maccentraleurope",     &udm_charset_macce},
  {"maccroation",          &udm_charset_maccroatian},
  {"maccyrillic",          &udm_charset_maccyrillic},
  {"macgreek",             &udm_charset_macgreek},
#ifdef HAVE_CHARSET_gujarati
  {"macgujarati",          &udm_charset_gujarati},
#endif
  {"machebrew",            &udm_charset_machebrew},
  {"macintosh",            &udm_charset_macroman},
  {"macisland",            &udm_charset_maciceland},
  {"macroman",             &udm_charset_macroman},
  {"macromania",           &udm_charset_macromania},
  {"macthai",              &udm_charset_macthai},
  {"macturkish",           &udm_charset_macturkish},
  {"ms-ansi",              &udm_charset_cp1252},
  {"ms-arab",              &udm_charset_cp1256},
  {"ms-cyr",               &udm_charset_cp1251},
  {"ms-ee",                &udm_charset_cp1250},
  {"ms-greek",             &udm_charset_cp1253},
  {"ms-hebr",              &udm_charset_cp1255},
  {"ms-turk",              &udm_charset_cp1254},
#ifdef HAVE_CHARSET_japanese
  {"ms_kanji",             &udm_charset_sjis},
  {"s-jis",                &udm_charset_sjis},
  {"shift-jis",            &udm_charset_sjis},
  {"shift_jis",            &udm_charset_sjis},
  {"sjis",                 &udm_charset_sjis},
#endif
  {"sys-int",              &udm_charset_sys_int},
  {"tactis",               &udm_charset_iso8859_11},
  {"tis-620",              &udm_charset_iso8859_11},
  {"tis620",               &udm_charset_iso8859_11},
#ifdef HAVE_CHARSET_tscii
  {"tscii",                &udm_charset_tscii},
#endif
#ifdef HAVE_CHARSET_japanese
  {"ujis",                 &udm_charset_eucjp},
#endif
  {"us",                   &udm_charset_usascii},
  {"us-ascii",             &udm_charset_usascii},
  {"utf-8",                &udm_charset_utf8},
  {"utf8",                 &udm_charset_utf8},
  {"viscii",               &udm_charset_viscii},
  {"viscii1.1-1",          &udm_charset_viscii},
  {"win-1251",             &udm_charset_cp1251},
  {"win1251",              &udm_charset_cp1251},
  {"winbaltrim",           &udm_charset_cp1257},
  {"windows-1250",         &udm_charset_cp1250},
  {"windows-1251",         &udm_charset_cp1251},
  {"windows-1252",         &udm_charset_cp1252},
  {"windows-1253",         &udm_charset_cp1253},
  {"windows-1254",         &udm_charset_cp1254},
  {"windows-1255",         &udm_charset_cp1255},
  {"windows-1256",         &udm_charset_cp1256},
  {"windows-1257",         &udm_charset_cp1257},
  {"windows-1258",         &udm_charset_cp1258},
  {"windows-874",          &udm_charset_cp874},
  {"windows-utf-8",        &udm_charset_utf8},
#ifdef HAVE_CHARSET_japanese
  {"x-euc-jp",             &udm_charset_eucjp},
#endif
  {"x-mac-cyrillic",       &udm_charset_maccyrillic},
#ifdef HAVE_CHARSET_japanese
  {"x-sjis",               &udm_charset_sjis},
#endif
  {NULL,0}
};



UDM_API(UDM_CHARSET *)
UdmGetCharSetByID(int id)
{
  UDM_CHARSET **c;
  
  for (c= built_charsets2; *c; c++)
    if (c[0]->id == id)
      return c[0];
  return NULL;
}

const char * UdmCharsetCanonicalName(const char * s)
{
  UDM_CHARSET *c= UdmGetCharSet(s);
  if(!c)return NULL;
  return c->name;
}

UDM_API(UDM_CHARSET *)
UdmGetCharSet(const char * name)
{
  int l,m,r,s;

  l= 0;
  s= r= sizeof(alias)/sizeof(UDM_CHARSET_ALIAS)-1;
  
  while(l < r)
  {
    m= (l+r)/2;
    if(strcasecmp(alias[m].name,name)<0)
      l=m+1;
    else
      r=m;
  }
  if(s==r)
    return(NULL);
  else
  {
    if(!strcasecmp(alias[r].name,name))
      return alias[r].cs;
    else  
      return NULL;
  }
  return NULL;
}
